#include <glut.h>
#include <stdio.h>


// These variables represent our current translation and rotation
float xOffset = 0.0f;
float yOffset = 0.0f;
float theta = 0.0f;


// The display function. It is called whenever the window needs
// redrawing (ie: overlapping window moves, resize, maximize)
// You should redraw your polygons here
void display(void) {

	// Clear the background
	glClear(GL_COLOR_BUFFER_BIT);

	// Reset the transformation matrix
	glLoadIdentity();

	// IMPORTANT: The last transformation command called in your
	// program is actually the first one applied to the vertices.
	glTranslatef(xOffset, yOffset, 0.0f);
	glTranslatef(0.1f, 0.1f, 0.0f);
	glRotatef(theta, 0.0f, 0.0f, 1.0f);
	glTranslatef(-0.1f, -0.1f, 0.0f);

	// Draw a red triangle  
	glColor3f(1,0,0);
	glBegin(GL_POLYGON);
		glVertex2f(0.1f,0.1f);
		glVertex2f(0.5f,0.1f);
		glVertex2f(0.1f,0.5f);
	glEnd();

	// Reset the transformation matrix
	glLoadIdentity();

	// Draw a blue line
	glColor3f(0,0,1);
	glBegin(GL_LINES);
		glVertex2f(0.9f,0.9f);
		glVertex2f(0.1f,0.1f);
	glEnd();

	// (Note that the origin is lower left corner)
	// (Note also that the window spans (0,1) )
	// Finish drawing, update the frame buffer
	glFlush();

	// Swap the buffers (remember we are using double buffers)
	glutSwapBuffers();
}


// This function is called whenever the window is resized. 
// Parameters are the new dimentions of the window
void    resize(int x,int y) {
	glViewport(0,0,x,y);
	printf("Resized to %d %d\n",x,y);
}


// This functions is called whenever the mouse is pressed or released
// button is a number 0 to 2 designating the button
// state is 1 for release 0 for press event
void    mouse(int button,int state,int x,int y) {
	printf("Mouse click at %d %d, button: %d, state %d\n",x,y,button,state);
}


// This function handles keypresses of "special" keys - in this case
// the arrows.
void processSpecialKeys(int key, int x, int y) {
	int mod;
	mod = glutGetModifiers();
	printf("Key pressed: %d with modifier %d\n", key, mod);
	switch(key) {
		case GLUT_KEY_LEFT:
			if (mod == GLUT_ACTIVE_CTRL) {
				theta += .5f;
			} else {
				xOffset -= .01f;
			}
			break;
		case GLUT_KEY_RIGHT:
			if (mod == GLUT_ACTIVE_CTRL) {
				theta -= .5f;
			} else {
				xOffset += .01f;
			}
			break;
		case GLUT_KEY_DOWN:
			yOffset -= .01f;
			break;
		case GLUT_KEY_UP:
			yOffset += .01f;
			break;			
	}

	// Requests a redraw - without this the display would be static
	glutPostRedisplay();
}


void initGL(void) {
	// Set projection type
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, 1, 0, 1, 0, 100);

	// Return to model view mode
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}



int main(int argc, char** argv) {

	// Initialize GLUT
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
	glutCreateWindow("Assignment 2 template");
	glutDisplayFunc(display);
	glutReshapeFunc(resize);
	glutMouseFunc(mouse);
	glutSpecialFunc(processSpecialKeys);

	// Initialize GL
	initGL();

	// Switch to main loop
	glutMainLoop();

	return 0;   
}